//#define STACK_SIZE 64

enum NTYPE
{
	NTYPE_INTEGER,
	NTYPE_OPERATOR
}

enum OPERATOR
{
	ParenthesisOpen = 0, // (
	ParenthesisClose,    // )
	OnesComplement,      // ~
	LogicalNot,          // !
	Multiplication,      // *
	Division,            // /
	Modulus,             // %
	Addition,            // +
	Subtraction,         // -
	LeftShift,           // <<
	RightShift,          // >>
	LessThan,            // <
	GreaterThan,         // >
	LessThanOrEqualTo,   // <=
	GreaterThanOrEqualTo,// >=
	Equality,            // ==
	Inequality,          // !=
	BitwiseAND,          // &
	BitwiseExclusiveOR,  // ^
	BitwiseInclusiveOR,  // |
	LogicalAND,          // &&
	LogicalOR,           // ||
	Conditional1,        // ?   e1?e2:e3
	Conditional2,        // :   e1?e2:e3
	Comma,               // ,   allowed only inside function(); ignored in eval()
	OperatorMax
}

struct STACKNODE
{
	NTYPE type;
	union
	{
		int value;       // case NTYPE_INTEGER
		OPERATOR optype; // case NTYPE_OPERATOR
		LPSTR FuncName;  // used if type=NTYPE_OPERATOR and optype should be ParenthesisOpen.
		                 // If so, optype will be reported as >65535
	}
	int srcIndex;
}

struct FUNCPARAMS
{
	int count;
	int param[0];
}

enum ALUERROR
{
	ALU_SUCCESS = 0,
	ALU_INVALIDTOKEN, // still success
	ALU_INVALIDTOKEN2,
	ALU_UNKNOWNVARIABLE,
	ALU_UNKNOWNFUNCTION,
	ALU_EMPTYPARENTHESES,
	ALU_STACKOVERFLOW,
	ALU_PARENTHESENOTOPENED,
	ALU_PARENTHESENOTCLOSED,
	ALU_UNKNOWNOPERATOR,
	ALU_UNHANDLEDOPERATOR,
	ALU_SYNTAXERROR,
	ALU_INTEGEREXPECTED,
	ALU_COMMAOUTSIDEFUNCTION,
	ALU_DIVISIONBYZERO,
	ALU_OPERATOREXPECTED,
	ALU_ERRORMAX
}
class ALU
{
	declare ALU();
	declare _ALU();
	// used to clear the internal stack
	declare empty();

	// high level method, returns number of bytes processed
	declare parse(byte *text, int *ppResult, BOOL *ppSuccess),int;
	// value: a digit or a operator
	declare add(string *pszValue, int srcIndex),BOOL;

//{ variables support
	declare virtual GetVariableValue(string name, int *ppValue),BOOL;
//}
//{ functions support
	declare virtual ExecuteFunction(string name, FUNCPARAMS *parameters, int *ppResult),BOOL;
//}
	declare GetError(ALUERROR *ppCode, LPSTR *ppDescription, int *ppCharacterIndex);

	declare push(STACKNODE *pNode),BOOL;
//	declare pop(STACKNODE *pNode),BOOL;  this method was not used

	declare AfterPushInteger();
	declare AfterPushOperator();

	declare IsOperator(int position,opt OPERATOR optype=-1),BOOL;
	declare IsInteger(int position),BOOL;
	declare GetAt(int position, STACKNODE *pNode),BOOL;
	declare SetAt(int position, STACKNODE *pNode),BOOL; // if (!pos) AfterPushInteger();
	declare DeleteAt(int position),BOOL;
	//
	declare Eval(opt int from=0,opt int to=-1),BOOL;
	// operator dispatchers; pos points to the operator: [pos+1]integer [pos]operator [pos-1]integer
	// each method checks only for parameter types: integer or operator.
	// e.g. fnAddition does not check if the operator is +
	// To track success/fail, you should compare m_top before with m_top after
	declare fnParenthesisClose(int pos);
	declare fnOnesComplement(int pos); // [pos]operator [pos-1]integer
	declare fnLogicalNot(int pos);     // [pos]operator [pos-1]integer
	declare fnMultiplication(int pos);
	declare fnDivision(int pos);
	declare fnModulus(int pos);
	declare fnAddition(int pos);       // supports a sign: 3*+3
	declare fnSubtraction(int pos);    // supports a sign: 3*-3
	declare fnLeftShift(int pos);
	declare fnRightShift(int pos);
	declare fnLessThan(int pos);
	declare fnGreaterThan(int pos);
	declare fnLessThanOrEqualTo(int pos);
	declare fnGreaterThanOrEqualTo(int pos);
	declare fnEquality(int pos);
	declare fnInequality(int pos);
	declare fnBitwiseAND(int pos);
	declare fnBitwiseExclusiveOR(int pos);
	declare fnBitwiseInclusiveOR(int pos);
	declare fnLogicalAND(int pos);
	declare fnLogicalOR(int pos);
	declare fnConditional(int pos);
	declare fnNegation(int pos);
	// validate int,operator,int; where pos = index of operator
	declare CheckTypes1(int pos),BOOL;
	// a optimization helper
	declare EvalPopParams(int pos byref, STACKNODE* x,STACKNODE* y);

	declare ReportError(ALUERROR code, int srcIndex);
	// debug
	declare dump();

	int  m_top; // pointer to first empty node on the stack
	int  m_stacksize;
	STACKNODE *m_stack;//[STACK_SIZE]; // m_stack[0] is the bottom; all indexes (pos,position,from,to) are relative to the top
	LPSTR m_pszCurrentFunction;
	// error
	ALUERROR m_errorCode;
	int      m_errorPosition; // character offset in string passed to parse()
}

#ifndef _alu_
extern g_aluHwndParent as HWND;
#endif

/*
after parse("1+2+3"); the local stack is:

m_stack[4] = '3';
m_stack[3] = '+';
m_stack[2] = '2';
m_stack[1] = '+';
m_stack[0] = '1';

GetAt(0) will return '3', because this is the top of stack
*/

#autodefine "OFF"

